"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var bs58 = require("bs58");
var util = require("ethereumjs-util");
var Web3 = require("web3");
var web3 = new Web3();
exports.toAscii = function (value) {
    return util.toAscii(value).replace(/\u0000/g, "");
};
exports.grantItemFromEvent = function (event) {
    return {
        role: exports.toAscii(event.role),
        resource: exports.toAscii(event.resource),
        action: exports.toAscii(event.action),
        attributes: event.attributes.map(exports.toAscii)
    };
};
exports.grantItemFromValues = function (values) {
    return {
        role: exports.toAscii(values[0]),
        resource: exports.toAscii(values[1]),
        action: exports.toAscii(values[2]),
        attributes: values[3].map(exports.toAscii)
    };
};
exports.permissionFromCanRoleActionResourceValues = function (values) {
    return {
        granted: values[0],
        resource: exports.toAscii(values[2]),
        attributes: values[0] ? ["*"] : [],
        _: {
            role: exports.toAscii(values[1]),
            resource: exports.toAscii(values[2]),
            attributes: values[0] ? ["*"] : [] // values[3].map(toAscii)
        }
    };
};
// https://blog.stakeventures.com/articles/smart-contract-terms
exports.hex2ipfshash = function (hash) {
    return bs58.encode(new Buffer("1220" + hash.slice(2), "hex"));
};
exports.ipfs2hex = function (ipfshash) {
    return "0x" + new Buffer(bs58.decode(ipfshash).slice(2)).toString("hex");
};
exports.convertValueToType = function (_valueType, _value) {
    // 'I' Encodes that this is IPLD, so we know to remove Qm (and add it back)
    if (_valueType === "I") {
        _value = exports.ipfs2hex(_value);
    }
    // Left padd ints and addresses for bytes32 equivalence of Solidity casting
    if (_valueType === "U" || _valueType === "A") {
        _value = util.bufferToHex(util.setLengthLeft(_value, 32));
    }
    return _value;
};
exports.getValueFromType = function (type, value) {
    switch (type) {
        case "A":
            return "0x" + value.split("0x000000000000000000000000")[1];
        case "U":
            return web3.toBigNumber(value).toNumber();
        case "B":
            return value;
        case "X":
            return exports.toAscii(value);
        case "I":
            return exports.hex2ipfshash(value);
    }
};
exports.guessTypeFromValue = function (value) {
    if (typeof value === "number") {
        return "U";
    }
    if (typeof value === "object") {
        return "I";
    }
    if (typeof value === "string") {
        if (util.isValidAddress(value)) {
            return "A";
        }
        if (value.length > 1 && value.substr(0, 2) == "0x") {
            return "X";
        }
        else {
            return "S";
        }
    }
    throw Error("unable to guess type of value: " + value);
};
exports.marshal = function (_eventType, _keyType, _valueType, _key, _value) {
    return {
        eventType: _eventType,
        keyType: _keyType,
        valueType: _valueType,
        key: exports.convertValueToType(_keyType, _key),
        value: exports.convertValueToType(_valueType, _value)
    };
};
exports.getUnmarshalledObjectFromValues = function (_id, _txOrigin, _created, _eventType, _keyType, _valueType, _key, _value) {
    _keyType = exports.toAscii(_keyType);
    _valueType = exports.toAscii(_valueType);
    _key = exports.getValueFromType(_keyType, _key);
    _value = exports.getValueFromType(_valueType, _value);
    return {
        id: _id.toNumber(),
        txOrigin: _txOrigin,
        created: _created.toNumber(),
        eventType: exports.toAscii(_eventType),
        keyType: _keyType,
        valueType: _valueType,
        key: _key,
        value: _value
    };
};
exports.unmarshal = function (eventArgs) {
    return exports.getUnmarshalledObjectFromValues(eventArgs.Id, eventArgs.TxOrigin, eventArgs.Created, eventArgs.EventType, eventArgs.KeyType, eventArgs.ValueType, eventArgs.Key, eventArgs.Value);
};
exports.getFSAFromFlat = function (flat) {
    return {
        type: flat.eventType,
        payload: (_a = {},
            _a[flat.key] = flat.value,
            _a),
        meta: {
            id: flat.id,
            created: flat.created,
            txOrigin: flat.txOrigin,
            keyType: flat.keyType,
            valueType: flat.valueType
        }
    };
    var _a;
};
exports.getFSAFromEventValues = function (_id, _txOrigin, _created, _eventType, _keyType, _valueType, _key, _value) {
    var flat = exports.getUnmarshalledObjectFromValues(_id, _txOrigin, _created, _eventType, _keyType, _valueType, _key, _value);
    return exports.getFSAFromFlat(flat);
};
exports.getFSAFromEventArgs = function (eventArgs) {
    var flat = exports.unmarshal(eventArgs);
    return exports.getFSAFromFlat(flat);
};
// http://stackoverflow.com/questions/19098797/fastest-way-to-flatten-un-flatten-nested-json-objects
exports.flatten = function (data) {
    var result = {};
    function recurse(cur, prop) {
        if (Object(cur) !== cur) {
            result[prop] = cur;
        }
        else if (Array.isArray(cur)) {
            for (var i = 0, l = cur.length; i < l; i++)
                recurse(cur[i], prop + "[" + i + "]");
            if (l == 0)
                result[prop] = [];
        }
        else {
            var isEmpty = true;
            for (var p in cur) {
                isEmpty = false;
                recurse(cur[p], prop ? prop + "." + p : p);
            }
            if (isEmpty && prop)
                result[prop] = {};
        }
    }
    recurse(data, "");
    return result;
};
exports.unflatten = function (data) {
    if (Object(data) !== data || Array.isArray(data))
        return data;
    var regex = /\.?([^.\[\]]+)|\[(\d+)\]/g, resultholder = {};
    for (var p in data) {
        var cur = resultholder, prop = "", m;
        while ((m = regex.exec(p))) {
            cur = cur[prop] || (cur[prop] = m[2] ? [] : {});
            prop = m[2] || m[1];
        }
        cur[prop] = data[p];
    }
    return resultholder[""] || resultholder;
};
//# sourceMappingURL=Common.js.map